import FS, { fstat } from 'node:fs'
import crypto from 'node:crypto'
import assert from 'node:assert'

const debug = process.env.debug || global?.debug

const log = console.log.bind(console)

const isJSON = s => {
  try{ return JSON.parse(s) } catch(e) {}
}

const get = async (url, options = {}, after = x=>x) => {
  if(!options.headers) options.headers = {}
  if(!options.headers.accept) options.headers.accept = 'application/json'
  if(global.debug) console.log('get()', url, options, after)
  const f = await fetch(`https://${url}`, options)
  const s = await f.text()
  return after(isJSON(s) || s)
}

const save = (o, fn) => {
  FS.writeFileSync(`./${fn}`, JSON.stringify(o, null, 2))
  return fn
}

const read = (fn) => {
  try {
    return FS.readFileSync(`./${fn}`, 'utf8')
  } catch (e) {}
}

const load = (fn) => {
  return isJSON(read(fn))
}

const exists = (fn) => {
  return FS.existsSync(fn)
}

const del = (fn) => {
  FS.unlinkSync(fn)
  return fn
}

const hash = (s) => {
   return crypto.createHash('sha256').update(s).digest('hex')
}

const te = (a, b) => {
   return assert.equal(a, b)
}

const tm = (a, b) => {
  return assert.match(a, b)
}

const wait = async (ms) => {
  return new Promise((r)=>setTimeout(r.bind('', 'wait over'), ms))
}

export { log, isJSON, get, save, read, load, exists, del, hash, te, tm, wait }
